package net.tittel.gridworldsim.server;

import net.tittel.gridworldsim.commons.TwoDimLocation;

/**
 * Objects made from this class represent a location on the grid.
 * @author Stefan Tittel <bugreports@tittel.net>
 */
public class GridLocation extends TwoDimLocation {

    /**
     * Constructs a new {@code GridLocation}.
     * @param x the x coordinate
     * @param y the y coordinate
     */
    public GridLocation(int x, int y) {
        super(x, y);
    }

    /**
     * Constructs a new empty/uninitialized {@code GridLocation}.
     */
    public GridLocation() {
        super();
    }

    /**
     * Constructs a new {@code GridLocation}.
     * @param twoDim the {@link TwoDimLocation} containing the two dimensional location
     */
    public GridLocation(TwoDimLocation twoDim) {
        this.setXY(twoDim.getX(), twoDim.getY());
    }

    /**
     * Checks if this {@code GridLocation} is a valid location in a given {@link GridWorld}.
     * @param gridWorld the {@link GridWorld} to check if this {@code GridLocation} is valid there
     * @return true if this {@code GridLocation} is valid for the given {@link GridWorld}, false otherwise
     */
    public boolean isOnGrid(GridWorld gridWorld) {
        if (this.getX() < 0 || this.getY() < 0 || this.getX() > gridWorld.getXDimension() - 1 || this.getY() > gridWorld.getYDimension() - 1) {
            return false;
        }
        return true;
    }

    /**
     * Checks if this {@code GridLocation} is describing the same location as another {@code GridLocation}.
     * @param obj the {@code Object} to check location equality for
     * @return true if the comparison {@code Object} is a {@code GridLocation} and locations are equal, otherwise false
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if ((obj == null) || this.getClass() != obj.getClass()) {
            return false;
        }

        GridLocation checkMe = (GridLocation) (obj);

        if (checkMe.getX() == this.getX() && checkMe.getY() == this.getY()) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Returns a hash code value for this {@code GridLocation}.
     * @return a hash code value for this {@code GridLocation}
     */
    @Override
    public int hashCode() {
        int hash = 7;
        hash = 67 * hash + this.getX();
        hash = 67 * hash + this.getY();
        return hash;
    }

    /**
     * Get the {@link GridCell} for this {@code GridLocation} in a {@link GridWorld}
     * @param gridWorld the {@link GridWorld}
     * @return the {@link GridCell} for this {@code GridLocation}
     */
    public GridCell getGridCell(GridWorld gridWorld) {
        return gridWorld.getGridCells()[this.getX()][this.getY()];
    }
}
